---
search:
  exclude: true
---
# 멀티 에이전트 오케스트레이션

오케스트레이션은 앱에서 에이전트가 흐르는 방식을 말합니다. 어떤 에이전트가 어떤 순서로 실행되며, 다음에 무엇을 할지 어떻게 결정할까요? 에이전트를 오케스트레이션하는 방법은 두 가지가 있습니다:

1. LLM이 결정하도록 하기: LLM의 지능을 사용해 계획하고 추론하여 다음에 취할 단계를 결정
2. 코드로 오케스트레이션하기: 코드로 에이전트의 흐름을 결정

이 패턴들은 혼합하여 사용할 수 있습니다. 각 방식에는 아래에 설명된 트레이드오프가 있습니다.

## LLM 기반 오케스트레이션

에이전트는 지침, 도구, 핸드오프로 무장한 LLM입니다. 이는 개방형 작업이 주어졌을 때, LLM이 도구를 사용해 행동하고 데이터를 획득하며, 핸드오프를 사용해 하위 에이전트에 작업을 위임함으로써 작업을 수행할 방법을 자율적으로 계획할 수 있음을 의미합니다. 예를 들어, 리서치 에이전트는 다음과 같은 도구를 갖출 수 있습니다:

- 웹 검색으로 온라인에서 정보 찾기
- 파일 검색과 검색으로 독점 데이터와 연결에서 탐색
- 컴퓨터 사용으로 컴퓨터에서 행동 수행
- 코드 실행으로 데이터 분석 수행
- 계획 수립, 보고서 작성 등에 능숙한 특화 에이전트로의 핸드오프

이 패턴은 작업이 개방형이고 LLM의 지능에 의존하고 싶을 때 적합합니다. 여기서 가장 중요한 전술은 다음과 같습니다:

1. 좋은 프롬프트에 투자하세요. 사용 가능한 도구, 사용하는 방법, 그리고 준수해야 하는 매개변수를 명확히 하세요.
2. 앱을 모니터링하고 반복적으로 개선하세요. 문제가 발생하는 지점을 확인하고 프롬프트를 개선하세요.
3. 에이전트가 자기 성찰하고 개선하도록 하세요. 예를 들어 루프에서 실행하여 스스로 비판하게 하거나, 오류 메시지를 제공해 개선하도록 하세요.
4. 모든 것을 잘하는 범용 에이전트보다 하나의 작업에 뛰어난 특화 에이전트를 두세요.
5. [평가 (evals)](https://platform.openai.com/docs/guides/evals)에 투자하세요. 이를 통해 에이전트를 훈련해 지속적으로 개선하고 작업에 더 능숙해지게 할 수 있습니다.

## 코드 기반 오케스트레이션

LLM 기반 오케스트레이션이 강력하긴 하지만, 코드 기반 오케스트레이션은 속도, 비용, 성능 측면에서 작업을 더 결정적이고 예측 가능하게 만듭니다. 여기서 흔한 패턴은 다음과 같습니다:

- [structured outputs](https://platform.openai.com/docs/guides/structured-outputs)를 사용해 코드로 검사할 수 있는 적절한 형식의 데이터를 생성. 예를 들어, 에이전트에게 작업을 몇 가지 카테고리로 분류하도록 요청한 다음, 카테고리에 따라 다음 에이전트를 선택할 수 있습니다.
- 한 에이전트의 출력을 다음 에이전트의 입력으로 변환해 여러 에이전트를 체이닝. 블로그 포스트 작성 같은 작업을 연구, 개요 작성, 본문 작성, 비판, 개선의 일련의 단계로 분해할 수 있습니다.
- 특정 기준을 통과한다고 평가자가 판단할 때까지, 작업을 수행하는 에이전트와 평가 및 피드백을 제공하는 에이전트를 `while` 루프로 실행
- `asyncio.gather` 같은 Python 기본 구성 요소를 통해 여러 에이전트를 병렬 실행. 서로 의존하지 않는 여러 작업이 있을 때 속도에 유용합니다.

[`examples/agent_patterns`](https://github.com/openai/openai-agents-python/tree/main/examples/agent_patterns)에서 여러 예제를 제공하고 있습니다.